﻿///<reference path='../declare/declare.ts' />

module Application {
    "use strict";

    var appView = Windows.UI.ViewManagement.ApplicationView;
    var nav = WinJS.Navigation;

    export var navigator: PageControlNavigator;

    interface PageControl {
        getAnimationElements: () => Element;
        updateLayout: (element: Element, viewState: Windows.UI.ViewManagement.ApplicationViewState, lastViewstate: Windows.UI.ViewManagement.ApplicationViewState) => void;
    }

    export class PageControlNavigator {
        home: string = "";
        _element: Element = null;
        _lastNavigationPromise: WinJS.Promise = WinJS.Promise.as();
        _lastViewstate: Windows.UI.ViewManagement.ApplicationViewState;

        constructor (element, options) {
            this._element = element || document.createElement("div");
            this._element.appendChild(this._createPageElement());

            this.home = options.home;
            this._lastViewstate = appView.value;

            nav.onnavigated = args => this._navigated(args);
            window.onresize = args => this._resized(args);

            document.body.onkeyup = args => this._keyupHandler(args);
            document.body.onkeypress = args => this._keypressHandler(args);
            document.body.onmspointerup = args => this._mspointerupHandler(args);

            navigator = this;
        }

        // This is the currently loaded Page object.
        get pageControl(): PageControl {
            return this.pageElement ? <PageControl>this.pageElement.winControl : null;
        }

        // This is the root element of the current page.
        get pageElement(): Element {
            return this._element.firstElementChild;
        }

        // Creates a container for a new page to be loaded into.
        _createPageElement(): Element {
            var element = document.createElement("div");
            element.style.width = "100%";
            element.style.height = "100%";
            return element;
        }

        // Retrieves a list of animation elements for the current page.
        // If the page does not define a list, animate the entire page.
        _getAnimationElements(): Element {
            if (this.pageControl && this.pageControl.getAnimationElements) {
                return this.pageControl.getAnimationElements();
            }
            return this.pageElement;
        }

        // Navigates back whenever the backspace key is pressed and
        // not captured by an input field.
        _keypressHandler(args: KeyboardEvent) {
            if (args.key === "Backspace") {
                nav.back();
            }
        }

        // Navigates back or forward when alt + left or alt + right
        // key combinations are pressed.
        _keyupHandler(args: KeyboardEvent) {
            if ((args.key === "Left" && args.altKey) || (args.key === "BrowserBack")) {
                nav.back();
            } else if ((args.key === "Right" && args.altKey) || (args.key === "BrowserForward")) {
                nav.forward();
            }
        }

        // This function responds to clicks to enable navigation using
        // back and forward mouse buttons.
        _mspointerupHandler(args: MSPointerEvent) {
            if (args.button === 3) {
                nav.back();
            } else if (args.button === 4) {
                nav.forward();
            }
        }

        // Responds to navigation by adding new pages to the DOM.
        _navigated(args: WinJS.Navigation.NavigationEvent) {
            var newElement = this._createPageElement();
            var parentedComplete;
            var parented = new WinJS.Promise(function (c) { parentedComplete = c; });

            this._lastNavigationPromise.cancel();

            this._lastNavigationPromise = WinJS.Promise.timeout().then(function () {
                return WinJS.UI.Pages.render(args.detail.location, newElement, args.detail.state, parented);
            }).then(function parentElement(control) {
                var oldElement = this.pageElement;
                if (oldElement.winControl && oldElement.winControl.unload) {
                    oldElement.winControl.unload();
                }
                this._element.appendChild(newElement);
                this._element.removeChild(oldElement);
                oldElement.innerText = "";
                this._updateBackButton();
                parentedComplete();
                WinJS.UI.Animation.enterPage(this._getAnimationElements()).done();
            }.bind(this));

            args.detail.setPromise(this._lastNavigationPromise);
        }

        // Responds to resize events and call the updateLayout function
        // on the currently loaded page.
        _resized(args: UIEvent) {
            if (this.pageControl && this.pageControl.updateLayout) {
                this.pageControl.updateLayout.call(this.pageControl, this.pageElement, appView.value, this._lastViewstate);
            }
            this._lastViewstate = appView.value;
        }

        // Updates the back button state. Called after navigation has
        // completed.
        _updateBackButton() {
            var backButton = this.pageElement.querySelector("header[role=banner] .win-backbutton");
            if (backButton) {
                backButton.onclick = () => nav.back();

                if (nav.canGoBack) {
                    backButton.removeAttribute("disabled");
                } else {
                    backButton.setAttribute("disabled", "disabled");
                }
            }
        }
    }

    WinJS.Utilities.markSupportedForProcessing(PageControlNavigator);
}
